use std::net::{Ipv4Addr, Ipv6Addr};

use super::{Name, Rtype};

#[derive(Clone)]
pub enum RData {
    A(A),
    Aaaa(Aaaa),
    Cname(Cname),
    Mx(Mx),
    Ns(Ns),
    Ptr(Ptr),
    Soa(Soa),
    Srv(Srv),
    Txt(Txt),
}

impl RData {
    pub fn rtype(&self) -> Rtype {
        match self {
            RData::A(_) => Rtype::A,
            RData::Aaaa(_) => Rtype::Aaaa,
            RData::Cname(_) => Rtype::Cname,
            RData::Mx(_) => Rtype::Mx,
            RData::Ns(_) => Rtype::Ns,
            RData::Ptr(_) => Rtype::Ptr,
            RData::Soa(_) => Rtype::Soa,
            RData::Srv(_) => Rtype::Srv,
            RData::Txt(_) => Rtype::Txt,
        }
    }
}

#[derive(Clone)]
pub struct A {
    pub address: Ipv4Addr,
}

#[derive(Clone)]
pub struct Aaaa {
    pub address: Ipv6Addr,
}

#[derive(Clone)]
pub struct Cname {
    pub target: Name,
}

#[derive(Clone)]
pub struct Mx {
    pub preference: u16,
    pub mail_exchanger: Name,
}

#[derive(Clone)]
pub struct Ns {
    pub target: Name,
}

#[derive(Clone)]
pub struct Ptr {
    pub target: Name,
}

#[derive(Clone)]
pub struct Soa {
    pub primary_server: Name,
    pub maintainer: Name,
    pub refresh: u32,
    pub retry: u32,
    pub expire: u32,
    pub minimum: u32,
    pub serial: u32,
}

#[derive(Clone)]
pub struct Srv {
    pub server: Name,
    pub port: u16,
    pub priority: u16,
    pub weight: u16,
}

#[derive(Clone)]
pub struct Txt {
    pub text: Vec<u8>,
}
